@@ -69,6 +69,7 @@ class NewRelicGQL(object):
                       name
                       authLabel
                       externalId
+                      metricCollectionMode
                     }
                   }
                 }
@@ -128,16 +129,6 @@ class NewRelicGQL(object):
         except KeyError:
             return None
 
-    def get_linked_account_by_name(self, name):
-        """
-        return a specific linked account of the New Relic account by name
-        """
-        accounts = self.get_linked_accounts()
-        try:
-            return next((a for a in accounts if a["name"] == name), None)
-        except KeyError:
-            return None
-
     def link_account(self, role_arn, account_name):
         """
         create a linked account (cloud integrations account)
@@ -145,7 +136,7 @@ class NewRelicGQL(object):
         """
         res = self.query(
             """
-            mutation ($accountId: Int!, $accounts: CloudLinkCloudAccountsInput!){
+            mutation ($accountId: Int!, $accounts: CloudLinkCloudAccountsInput!) {
               cloudLinkAccount (accountId: $accountId, accounts: $accounts) {
                 linkedAccounts {
                   id
@@ -178,8 +169,8 @@ class NewRelicGQL(object):
         """
         res = self.query(
             """
-            mutation ($accountId: Int!, $accounts: CloudUnlinkCloudAccountsInput!) {
-              cloudUnLinkAccount (accountId: $accountId, accounts: $accounts) {
+            mutation ($accountId: Int!, $accounts: [CloudUnlinkAccountsInput!]!) {
+              cloudUnlinkAccount (accountId: $accountId, accounts: $accounts) {
                 unlinkedAccounts {
                   id
                   name
@@ -192,9 +183,9 @@ class NewRelicGQL(object):
             }
             """,
             accountId=self.account_id,
-            accounts={"linkedAccountId": linked_account_id},
+            accounts=[{"linkedAccountId": linked_account_id}],
         )
-        if "errors" in res:
+        if "errors" in res and res["errors"]:
             failure(
                 "Error while unlinking account with New Relic:\n%s"
                 % "\n".join([e["message"] for e in res["errors"] if "message" in e])
@@ -340,7 +331,9 @@ def validate_gql_credentials(input):
         raise click.BadParameter(
             "Could not authenticate with New Relic. Check that your New Relic Account "
             "ID and API Key are valid and try again.",
+            ctx=None,
             param="nr_api_key",
+            param_hint="New Relic User API Key",
         )
 
 
@@ -356,7 +349,9 @@ def retrieve_license_key(gql):
         raise click.BadParameter(
             "Could not retrieve license key from New Relic. Check that your New Relic "
             "Account ID is valid and try again.",
+            ctx=None,
             param="nr_account_id",
+            param_hint="New Relic Account ID",
         )
 
 
@@ -407,6 +402,16 @@ def enable_lambda_integration(gql, input, linked_account_id):
             "[%d] in New Relic account [%d]." % (linked_account_id, input.nr_account_id)
         )
         return False
+
+    metric_collection_mode = account["metricCollectionMode"]
+
+    if metric_collection_mode == "PUSH":
+        success(
+            "Cloud integrations account [%s] is using CloudWatch Metric Streams"
+            % (input.nr_account_id)
+        )
+        return True
+
     is_lambda_enabled = gql.is_integration_enabled(linked_account_id, "lambda")
     if is_lambda_enabled:
         success(
