@@ -2,14 +2,29 @@
 
 import botocore
 import click
+import enquiries
 import json
+import re
 import requests
 
+
 from newrelic_lambda_cli import api, subscriptions, utils
-from newrelic_lambda_cli.cliutils import failure, success
+from newrelic_lambda_cli.cliutils import failure, success, warning
 from newrelic_lambda_cli.functions import get_function
-from newrelic_lambda_cli.integrations import _get_license_key_policy_arn
+from newrelic_lambda_cli.integrations import _get_license_key_outputs
 from newrelic_lambda_cli.types import LayerInstall, LayerUninstall
+from newrelic_lambda_cli.utils import catch_boto_errors
+
+
+NEW_RELIC_ENV_VARS = (
+    "NEW_RELIC_ACCOUNT_ID",
+    "NEW_RELIC_EXTENSION_SEND_FUNCTION_LOGS",
+    "NEW_RELIC_LAMBDA_EXTENSION_ENABLED",
+    "NEW_RELIC_LAMBDA_HANDLER",
+    "NEW_RELIC_LICENSE_KEY",
+    "NEW_RELIC_LOG_ENDPOINT",
+    "NEW_RELIC_TELEMETRY_ENDPOINT",
+)
 
 
 def index(region, runtime):
@@ -21,6 +36,25 @@ def index(region, runtime):
     return layers_response.get("Layers", [])
 
 
+def layer_selection(available_layers, runtime):
+    selected_layer = []
+    if len(available_layers) > 1:
+        layerOptions = []
+        for layer in available_layers:
+            layerOptions.append(layer["LatestMatchingVersion"]["LayerVersionArn"])
+
+        response = enquiries.choose(
+            "Discovered layers for runtime {0}".format(runtime), layerOptions
+        )
+        success("Layer {0} selected".format(response))
+        selected_layer.append(response)
+    else:
+        selected_layer.append(
+            available_layers[0]["LatestMatchingVersion"]["LayerVersionArn"]
+        )
+    return selected_layer
+
+
 def _add_new_relic(input, config, nr_license_key):
     assert isinstance(input, LayerInstall)
 
@@ -37,6 +71,10 @@ def _add_new_relic(input, config, nr_license_key):
     handler = config["Configuration"]["Handler"]
     runtime_handler = utils.RUNTIME_CONFIG.get(runtime, {}).get("Handler")
 
+    if "java" in runtime:
+        postfix = input.java_handler_method or "handleRequest"
+        runtime_handler = runtime_handler + postfix
+
     existing_newrelic_layer = [
         layer["Arn"]
         for layer in config["Configuration"].get("Layers", [])
@@ -57,10 +95,10 @@ def _add_new_relic(input, config, nr_license_key):
         if not layer["Arn"].startswith(utils.get_arn_prefix(aws_region))
     ]
 
-    new_relic_layers = []
+    new_relic_layer = []
 
     if input.layer_arn:
-        new_relic_layers = [input.layer_arn]
+        new_relic_layer = [input.layer_arn]
     else:
         # discover compatible layers...
         available_layers = index(aws_region, runtime)
@@ -72,25 +110,7 @@ def _add_new_relic(input, config, nr_license_key):
             )
             return False
 
-        # TODO: MAke this a layer selection screen
-        if len(available_layers) > 1:
-            message = ["Discovered layers for runtime (%s)" % runtime]
-            for layer in available_layers:
-                message.append(
-                    "%s\t%s"
-                    % (
-                        layer["LatestMatchingVersion"]["LayerVersionArn"],
-                        layer.get("Description", ""),
-                    )
-                )
-            message.append(
-                "\nMultiple layers found. Pass --layer-arn to specify layer ARN"
-            )
-            raise click.UsageError("\n".join(message))
-
-        new_relic_layers = [
-            available_layers[0]["LatestMatchingVersion"]["LayerVersionArn"]
-        ]
+        new_relic_layer = layer_selection(available_layers, runtime)
 
     update_kwargs = {
         "FunctionName": config["Configuration"]["FunctionArn"],
@@ -99,10 +119,14 @@ def _add_new_relic(input, config, nr_license_key):
             .get("Environment", {})
             .get("Variables", {})
         },
-        "Layers": new_relic_layers + existing_layers,
+        "Layers": new_relic_layer + existing_layers,
     }
 
-    # Only used by Python and Node.js runtimes
+    # We don't want to modify the handler if the NewRelicLambdaExtension layer has been selected
+    if any("NewRelicLambdaExtension" in s for s in new_relic_layer):
+        runtime_handler = None
+
+    # Only used by Python, Node.js and Java runtimes not using the NewRelicLambdaExtension layer
     if runtime_handler:
         update_kwargs["Handler"] = runtime_handler
 
@@ -115,19 +139,23 @@ def _add_new_relic(input, config, nr_license_key):
     if runtime_handler and handler != runtime_handler:
         update_kwargs["Environment"]["Variables"]["NEW_RELIC_LAMBDA_HANDLER"] = handler
 
-    if input.enable_extension:
+    if input.enable_extension and not utils.supports_lambda_extension(runtime):
+        warning(
+            "The %s runtime for %s does not support Lambda Extensions, reverting to a "
+            "CloudWatch Logs based ingestion. Make sure you run `newrelic-lambda "
+            "integrations install` command to install the New Relic log ingestion "
+            "function and `newrelic-lambda subscriptions install` to create the log "
+            "subscription filter." % (runtime, config["Configuration"]["FunctionName"])
+        )
+
+    if input.enable_extension and utils.supports_lambda_extension(runtime):
         update_kwargs["Environment"]["Variables"][
             "NEW_RELIC_LAMBDA_EXTENSION_ENABLED"
         ] = "true"
 
-        if input.enable_extension_function_logs:
-            update_kwargs["Environment"]["Variables"][
-                "NEW_RELIC_EXTENSION_SEND_FUNCTION_LOGS"
-            ] = "true"
-        else:
-            update_kwargs["Environment"]["Variables"][
-                "NEW_RELIC_EXTENSION_SEND_FUNCTION_LOGS"
-            ] = "false"
+        update_kwargs["Environment"]["Variables"][
+            "NEW_RELIC_EXTENSION_SEND_FUNCTION_LOGS"
+        ] = ("true" if input.enable_extension_function_logs else "false")
 
         if input.nr_region == "staging":
             update_kwargs["Environment"]["Variables"][
@@ -149,6 +177,7 @@ def _add_new_relic(input, config, nr_license_key):
     return update_kwargs
 
 
+@catch_boto_errors
 def install(input, function_arn):
     assert isinstance(input, LayerInstall)
 
@@ -159,7 +188,24 @@ def install(input, function_arn):
         failure("Could not find function: %s" % function_arn)
         return False
 
-    policy_arn = _get_license_key_policy_arn(input.session)
+    nr_account_id, policy_arn = _get_license_key_outputs(input.session)
+    # If a managed secret exists but it was created with a different NR account id and license key
+    # We want to notify the user and point them to documentation on how to proceed.
+    if (
+        policy_arn
+        and nr_account_id != str(input.nr_account_id)
+        and not input.nr_api_key
+    ):
+        raise click.UsageError(
+            "A managed secret already exists in this region for New Relic account {0}. "
+            "Creating one managed secret per region is currently supported via the cli.\n"
+            "To set up an additional secret for New Relic account {1} see our docs:\n{2}.\n"
+            "Or you can re-run this command with "
+            "`--nr-api-key` argument with your New Relic API key to set your license "
+            "key in a NEW_RELIC_LICENSE_KEY environment variable instead.".format(
+                nr_account_id, input.nr_account_id, utils.NR_DOCS_ACT_LINKING_URL
+            )
+        )
     if input.enable_extension and not policy_arn and not input.nr_api_key:
         raise click.UsageError(
             "In order to use `--enable-extension`, you must first run "
@@ -172,14 +218,18 @@ def install(input, function_arn):
         )
 
     nr_license_key = None
-    if not policy_arn and input.nr_api_key and input.nr_region:
+    if (
+        not policy_arn
+        or nr_account_id != str(input.nr_account_id)
+        and input.nr_api_key
+        and input.nr_region
+    ):
         gql = api.validate_gql_credentials(input)
         nr_license_key = api.retrieve_license_key(gql)
 
     update_kwargs = _add_new_relic(input, config, nr_license_key)
-
-    if not update_kwargs or not isinstance(update_kwargs, dict):
-        return False
+    if isinstance(update_kwargs, bool):
+        return update_kwargs
 
     try:
         res = client.update_function_configuration(**update_kwargs)
@@ -220,6 +270,11 @@ def _remove_new_relic(input, config):
 
     handler = config["Configuration"]["Handler"]
 
+    # For java runtimes we need to remove the method name before
+    # validating because method names are variable
+    if "java" in runtime:
+        handler = handler.split("::", 1)[0] + "::"
+
     # Detect non-New Relic handler and error if necessary.
     if not utils.is_valid_handler(runtime, handler):
         failure(
@@ -236,14 +291,6 @@ def _remove_new_relic(input, config):
         .get("NEW_RELIC_LAMBDA_HANDLER")
     )
 
-    if not env_handler:
-        failure(
-            "New Relic installation (via layers) not auto-detected for the specified "
-            "function '%s'. Environment variable NEW_RELIC_LAMBDA_HANDLER not found."
-            % config["Configuration"]["FunctionArn"]
-        )
-        return False
-
     # Delete New Relic env vars
     config["Configuration"]["Environment"]["Variables"] = {
         key: value
@@ -251,7 +298,7 @@ def _remove_new_relic(input, config):
         .get("Environment", {})
         .get("Variables", {})
         .items()
-        if not key.startswith("NEW_RELIC")
+        if key not in NEW_RELIC_ENV_VARS
     }
 
     # Remove New Relic layers
@@ -263,12 +310,13 @@ def _remove_new_relic(input, config):
 
     return {
         "FunctionName": config["Configuration"]["FunctionArn"],
-        "Handler": env_handler,
+        "Handler": env_handler if env_handler else config["Configuration"]["Handler"],
         "Environment": config["Configuration"]["Environment"],
         "Layers": layers,
     }
 
 
+@catch_boto_errors
 def uninstall(input, function_arn):
     assert isinstance(input, LayerUninstall)
 
@@ -281,8 +329,8 @@ def uninstall(input, function_arn):
 
     update_kwargs = _remove_new_relic(input, config)
 
-    if not update_kwargs:
-        return False
+    if isinstance(update_kwargs, bool):
+        return update_kwargs
 
     try:
         res = client.update_function_configuration(**update_kwargs)
@@ -293,7 +341,7 @@ def uninstall(input, function_arn):
         )
         return False
     else:
-        policy_arn = _get_license_key_policy_arn(input.session)
+        _, policy_arn = _get_license_key_outputs(input.session)
         if policy_arn:
             _detach_license_key_policy(
                 input.session, config["Configuration"]["Role"], policy_arn
